use super::{Context, Error};

use super::utils::serenity;

#[allow(clippy::unused_async)]
#[poise::command(
    slash_command,
    subcommands("add_channel", "list_channels"),
    required_permissions = "ADMINISTRATOR"
)]
pub async fn logging(_ctx: Context<'_>) -> Result<(), Error> {
    Ok(())
}

#[poise::command(slash_command, aliases("add-channel"))]
pub async fn add_channel(
    ctx: Context<'_>,
    #[description = "New logging channel"] channel: serenity::Channel,
) -> Result<(), Error> {
    let channel_id = channel.id();
    let response = match ctx.guild_id() {
        None => "Error: Could not determine the guild's ID".to_owned(),
        Some(guild_id) => {
            match ctx
                .data()
                .database
                .set_logging_channel(guild_id, channel_id)
                .await
            {
                Ok(()) => format!(
                    "Added channel <#{channel_id}> as a logging channel"
                ),
                Err(e) => {
                    if let Some(db_error) = e.as_database_error() {
                        if db_error.is_unique_violation() {
                            format!("Channel <#{channel_id}> is already a logging channel")
                        } else {
                            format!("Error: {e:?}")
                        }
                    } else {
                        format!(
                            "Something bad happened with the database: {e:?}"
                        )
                    }
                }
            }
        }
    };
    ctx.say(response).await?;
    Ok(())
}

#[poise::command(slash_command, aliases("list-channels"))]
pub async fn list_channels(ctx: Context<'_>) -> Result<(), Error> {
    let response = match ctx.guild_id() {
        None => "Error: Could not determine the guild's ID".to_owned(),
        Some(guild_id) => {
            match ctx.data().database.get_logging_channel(guild_id).await {
                Ok(channels) => {
                    if channels.is_empty() {
                        "No channels registered as loggers".to_owned()
                    } else {
                        format!(
                            "Here are the channels currently set as loggers:\n{}",
                            channels
                                .iter()
                                .map(|channel| format!("- <#{channel}>"))
                                .collect::<Vec<String>>()
                                .join("\n")
                        )
                    }
                }
                Err(e) => format!("Could not retrieve loggers: {e:?}"),
            }
        }
    };
    ctx.say(response).await?;
    Ok(())
}
